const io = require('socket.io');
const AuthService = require('./AuthService');
const UpService = require('./UpService');
const adqGetDashboardFiliais = require('../model/adqGetDashboardFiliais');
const deliveryController = require('../controller/DeliveryController');
const saleController = require('../controller/SaleController');
const sefazController = require('../controller/SefazController');
const intervalMonitoring = require('../functions/IntervalMonitoring');
const paginacao = require('../functions/Paginacao');
const updaterController = require('../controller/UpdaterController');

class SocketService {
  async start(http) {
    this.clientlist = {};
    this.browserlist = {};
    this.slavelist = await this.getDashboardFiliais();
    this.defaultInitTimeout = 5000;
    this.intervals = []; // Registrar is intervals
    this.conexoes = []; // Registrar os portais

    this.parametersError = [];
    this.vendasError = [];
    this.satError = [];
    this.io = io(http, { cors: '*' });

    this.io.on('connection', (socket) => {
      this.socket = socket;
      console.log('newconnection: ', socket.id);
      socket.emit('updateVersion', { serviceName: 'xxx' });

      /**
       * Atualiza informações do Dashboard, inclusive o ID do socket e registra a sala
       */
      socket.on('join', (room) => {
        this.slavelist.filter((filial) => {
          if(filial.id === room.id){
            Object.assign(filial, room);
          }
          return filial;
        });
        socket.join(room.room);
        // console.log('Room: ', socket.rooms);
      });

      /**
       * Teste de sala
       * Envia para a sala 1_9999
       * emita saleSlave para 1_9999
       */
      // this.intervals.push({
      //   id: socket.id,
      //   interval: setInterval(() => {
      //     socket.emit("joinedroom", {
      //       ID: socket.id
      //     }, '1_9999');
      //   }, 5000)
      // });

      /** Envia para o sala NRORG */
      // this.slavelist.forEach((filial) => {
      //   setInterval(() => {
      //     socket.emit("saleSlave", {
      //       ID: socket.id
      //     }, `${filial.nrorg}_${filial.id}`)
      //   }, 1000);
      // });


      /** Garante que o portal está conectado */
      socket.on('ping', (data) => {
        const clintId = (data.id)? data.id : data;
        if(data.filial) {
          /** se for loja, atualiza os dados e avise o portal */
          this.slavelist.filter((filial) => {
            if(filial.id === data.filial) {
              /** Merge de info */
              Object.assign(filial, {socketId: data.id, situacao: 'Ativo'});
              /** Atualiza o Portal */
              const portalList = Object.keys(this.clientlist).map((item) => {
                return this.clientlist[item].id; 
              });
              this.io.to(portalList).emit('login', {
                text: 'UNIDADE CONECTADA',
                error: false,
                message: {
                  data: [filial],
                },
              });
            }
            return filial;
          });
        }

        /** Confere as conexões */
        this.conexoes.filter((portais) => {
          if(portais.id === clintId){
            Object.assign(portais, {time: new Date().getTime()});
          }
          return portais;
        });
      });

      /** Remove conexões anativas */
      this.intervals.push({
        id: socket.id,
        interval: setInterval(() => {
          console.log('Conectado: ', socket.id, this.conexoes);
          this.conexoes.forEach((portais) => {
            if((portais.time + 1200000) < new Date().getTime()){
              /** Client não atualizou a conexão, mateo */
              // socket.disconnect();
              this.io.sockets.sockets.forEach((socket) => {
                // console.log('Deveria desconectar o: ', socket.id);
                if(socket.id === portais.id) {
                  // console.log('Desconectei o: ', socket.id);
                  socket.disconnect(true);
                }
              });
            }
          });
        }, 20000)
      });
      /** Garante que o portal está conectado */

      socket.on('login', (data) => {
        console.log('Login: ');
        this.login(socket, data);
      });

      /**
       * Exportação de vendas
       */
      socket.on('iniciaExportacao', (data) => {
        try {
          data.payload = JSON.parse(data.payload);
          UpService.setInsereServidor(data.CDFILIAL, data.CDCAIXA, data.CDLOJA, data.payload);
        } catch (e) {}
      });

      /**
       * Importação
       */
      socket.on('importar', (data) => {
        console.log('importar', data);
        let target = this.slavelist[socket.email][data.CDFILIAL];
        this.io
          .to(target.id)
          .emit('command', { type: 'import', data: { dataUrl: '/API/getStructure' } });
      });

      socket.on('confirmation', (data) => {
        // console.log(data);
      });

      /**
       * Desconet do socket
       */
      socket.on('disconnect', () => {
        /** Limpa os Intervals */
        this.intervals.forEach((interval) => {
          if(interval.id === socket.id) {
            clearInterval(interval.interval);
            // delete this.intervals[interval];
          }
        });
        const indIntervals = this.intervals.findIndex((item) => {
          return item.id === socket.id;
        });
        this.intervals.splice(indIntervals);

        /** Limpa as Conexões (Portais e Lojas) */
        // this.conexoes = this.conexoes.filter((portais) => portais.id === socket.id);
        const indConexoes = this.conexoes.findIndex((item) => {
          return item.id === socket.id;
        });
        this.conexoes.splice(indConexoes);

        const subsidiaryIndex = this.slavelist.findIndex((item) => {
          return item.socketId === socket.id;
        });

        if (this.slavelist[subsidiaryIndex]) {
          this.slavelist[subsidiaryIndex].situacao = 'Inativa';

          const portalList = Object.keys(this.clientlist).map((item) => {
            return this.clientlist[item].id;
          });
          if (portalList.length) {
            this.io.to(portalList).emit('login', {
              text: 'UNIDADE DESCONECTADA',
              error: false,
              message: {
                data: this.slavelist[subsidiaryIndex],
              },
            });
          }
        }
      });

      /**
       * Socket Portal
       * Socket de Vendas
       */
      socket.on('sales', (data) => {
        console.log('Portal Vendas: ', new Date().getSeconds());
        // Executar a primeira vez mais rápido quando abrir o Grid
        setTimeout(() => {
          /** Solicitação para as Lojas */
          // this.io.to(idsLojas(this.slavelist)).emit('saleSlaveError', data);
          doSales(this.vendasError, this.io);
        }, this.defaultInitTimeout);

        this.intervals.push({
          id: socket.id,
          interval: setInterval(() => {
            // /** Solicitação para as Lojas */
            // this.io.to(idsLojas(this.slavelist)).emit('saleSlaveError', data);
            doSales(this.vendasError, this.io);
          },
          intervalMonitoring(data))
        });
      });

      function doSales(vendasError, io) {
        saleController.salesSlave(vendasError).then((response) => {
          console.log('Portal responde Vendas: ', new Date().getSeconds());
          io.to(socket.id).emit('sales', {
            text: 'VENDAS',
            error: false,
            message: {
              data: response,
            },
          });
        }).catch((error) => {
          console.log(error);
        });
      }

      /**
       * Socket Portal
       * Socket de Delivery
       */
      socket.on('salesDelivery', (data) => {
        console.log('Portal Delivery: ', new Date().getSeconds());
        // Executar a primeira vez mais rápido quando abrir o Grid
        setTimeout(() => {
          doSalesDelivery(data, this.io);
        }, this.defaultInitTimeout);

        this.intervals.push({
          id: socket.id,
          interval: setInterval(() => {
              doSalesDelivery(data, this.io).then((response) => {
                response.map((response) => {
                  /** Atualiza o this.slavelist */
                  this.slavelist.filter((filial) => {
                    if(filial.id === response.id){
                      Object.assign(filial, {delivery: response.delivery});
                    }
                    return filial;
                  });
                });
              });
            }, intervalMonitoring(data))
          });
      });

      function doSalesDelivery(data, io) {
        return deliveryController.getAssistantDeliveryList(data).then((response) => {
          console.log('Portal responde Delivery: ', new Date().getSeconds());
          /** Lista de vendas Delivery */
          io.to(socket.id).emit('salesDelivery', {
            text: 'DELIVERY',
            error: false,
            message: {
              data: response,
            },
          });
          return response;
        }).catch((error) => {
          console.log('Erro foi: ', error);
        });
      }

      /**
       * Socket Portal
       * Socket de Sat
       */
      socket.on('sat', (data) => {
        /** Conectar com a Loja */
        if (data.socketClient !== undefined) {
          this.io.to(data.socketClient).emit('satSlave', data);
        }

        // Executar a primeira vez mais rápido quando abrir o Grid
        setTimeout(() => {
          /** Solicitação para as Lojas */
          // this.io.to(idsLojas(this.slavelist)).emit('satSlave', data);
          doSat(this.io, this.satError);
        }, this.defaultInitTimeout);

        this.intervals.push({
          id: socket.id,
          interval: setInterval(() => {
            console.log('Portal Sat: ', new Date().getSeconds());
            /** Solicitação para as Lojas */
            // this.io.to(idsLojas(this.slavelist)).emit('satSlave', data);
            doSat(this.io, this.satError);
          }, intervalMonitoring(data))
        });
      });

      function doSat(io, satError) {
        console.log('Portal responde SAT: ', new Date().getSeconds());
        io.to(socket.id).emit('sat', {
          text: 'SAT',
          error: false,
          message: {
            data: satError,
          },
        });
      }

      /**
       * Socket Portal
       * Socket Update
       */
      socket.on('updater', (data) => {
        /** Conectar com a Loja */
        // if (data.socketClient !== undefined) {
        //   this.io.to(data.socketClient).emit('satSlave', data);
        // }

        updaterController.getUnidades().then((result) => {
          const newItens = [];
          result.forEach((results) => {
            updaterController.getVersaoAtual(results.IDUPDATERSISTEMAS).then((resultss) => {
              results.VERSAOATUAL = resultss;
              newItens.push(results);
            });
          });
          setInterval(() => {
            this.io.to(socket.id).emit('updater', {
              text: 'UPDATE',
              error: false,
              message: {
                data: newItens,
              },
            });
          }, intervalMonitoring(data));
        }).catch((err) => {
          console.log(err);
        });
      });

      socket.on('updaterHistorico', (data) => {
        updaterController.getHistorico(data).then((result) => {
          this.io.to(socket.id).emit('updaterHistorico', {
            text: 'UPDATE',
            error: false,
            message: {
              data: result,
            },
          });
        });
      });

      socket.on('updaters', (data) => {
        if (data.tipo === 'up') {
          updaterController.updaterSistemas(data).then((result) => {
            console.log(result);
          }).catch((err) => {
            console.log(err);
          });
        } else if (data.tipo === 'down') {
          updaterController.getUnidades().then((result) => {
            console.log(result);
          }).catch((err) => {
            console.log(err);
          });
        } else if (data.tipo === 'historico') {
          updaterController.getUnidades().then((result) => {
            console.log(result);
          }).catch((err) => {
            console.log(err);
          });
        } else if (data.tipo === 'install') {
          updaterController.getUnidades().then((result) => {
            console.log(result);
          }).catch((err) => {
            console.log(err);
          });
        }
      });

      /**
       * Socket Portal
       * Socket de Parametros
       */
      socket.on('parameters', (data) => {
        console.log('Portal Parametros: ', new Date().getSeconds());
        // Executar a primeira vez mais rápido quando abrir o Grid
        setTimeout(() => {
          /** Solicitação para as Lojas */
          // this.io.to(idsLojas(this.slavelist)).emit('parametersSlave', data);
          doParameters(this.io, this.parametersError);
        }, this.defaultInitTimeout);

        this.intervals.push({
          id: socket.id,
          interval: setInterval(() => {
            /** Solicitação para as Lojas */
            // this.io.to(idsLojas(this.slavelist)).emit('parametersSlave', data);
            doParameters(this.io, this.parametersError);
          }, intervalMonitoring(data))
        });
      });

      function doParameters(io, parametersError) {
        console.log('Portal responde Parametros: ', new Date().getSeconds());
        io.to(socket.id).emit('parameters', {
          text: 'PARAMETERS',
          error: false,
          message: {
            data: parametersError,
          },
        });
      }

      /**
       * Socket Portal
       * Socket de Sefaz
       */
      socket.on('sefaz', (data) => {
        console.log('Portal Sefaz: ', new Date().getSeconds());
        // Executar a primeira vez mais rápido quando abrir o Grid
        setTimeout(() => {
          doSefaz(data, this.io);
        }, this.defaultInitTimeout);

        this.intervals.push({
          id: socket.id,
          interval: setInterval(() => {
            doSefaz(data, this.io).then((response) => {
              response.map((response) => {
                /** Atualiza o this.slavelist */
                this.slavelist.filter((filial) => {
                  if(filial.id === response.id){
                    let status = true;
                    if (response.idnormal === 'CINZA' && response.idscan === 'CINZA' && response.idsvc === 'CINZA' ) {
                      status = true;
                    } else {
                      status = false;
                    }
                    Object.assign(filial, {sefaz: status});
                  }
                  return filial;
                });
              });
            });
          }, intervalMonitoring(data))
        });
      });

      function doSefaz(data, io) {
        return sefazController.getSefaz(data).then((response) => {
          console.log('Portal responde Sefaz: ', new Date().getSeconds());
          io.to(socket.id).emit('sefaz', {
            text: 'SEFAZ',
            error: false,
            message: {
              data: response,
            },
          });
          return response;
        }).catch((error) => {
          console.log('Erro foi: Sefaz | ', error);
        });
      }
    });

    /** Retorna Array com todos os socketId e Salas das lojas */
    function idsLojas(clientlist) {
      let ids = [];
      Object.keys(clientlist).map((item) => {
        if(clientlist[item].socketId){
          /** Por iD */
          ids.push(clientlist[item].socketId);
          /** Por Sala */
          ids.push(`${clientlist[item].nrorg}_${clientlist[item].id}`);
        }
      });
      return ids;
    }
  }

  import(socket, data) {}

  login(socket, data) {
    // Não é uma filial que está conectando e sim um portal
    if (!data.CDFILIAL) {
      // TODO Autenticar esse cliente, senão desconectar ele

      let auth = AuthService.autenticar(data.email, data.password)
        .then((result) => {
          // Se estiver autenticado coloca ele na lista simples de controle
          socket.email = data.email;

          this.clientlist[data.email] = socket;
          this.io
            .to(socket.id)
            .emit('confirmLogin', { error: false, message: 'Você está conectado', result: result });

          if (!data.slave) {
            this.conexoes.push({
              id: socket.id,
              time: new Date().getTime(),
            });
            if (this.slavelist === undefined) {
              this.getDashboardFiliais(data).then((dashboard) => {
                this.io.to(socket.id).emit('login', {
                  text: 'PORTAL CONECTADO',
                  error: false,
                  message: dashboard,
                  result,
                });
              });
            } else {
              this.io.to(socket.id).emit('login', {
                text: 'PORTAL CONECTADO',
                error: false,
                message: this.slavelist,
                result,
              });
            }
          }
        })
        .catch((error) => {
          this.io.to(socket.id).emit('message', 'Você não é um usuário autorizado!');
          socket.disconnect();
        });
    } else {
      // Uma filial se coneta ao master
      // TODO Autenticar esse cliente, senão desconectar ele
      let auth = AuthService.autenticar(data.email, '', data.CDFILIAL)
        .then((result) => {
          // Se estiver autenticado coloca ele na lista simples de controle
          socket.email = data.email;
          socket.CDFILIAL = data.CDFILIAL;

          const subsidiaryIndex = this.slavelist.findIndex((item) => {
            return item.codigo === data.CDFILIAL;
          });

          if (this.slavelist[subsidiaryIndex]) {
            this.slavelist[subsidiaryIndex].situacao = 'Ativo';
            this.slavelist[subsidiaryIndex].socketId = socket.id;

            /** Refistra a conexão */
            this.conexoes.push({
              id: socket.id,
              time: new Date().getTime(),
            });

            const portalList = Object.keys(this.clientlist).map((item) => {
              return this.clientlist[item].id;
            });

            /**
             * Socket Loja
             */
            this.io.to(socket.id).emit('message', result);
            this.io
              .to(socket.id)
              .emit('command', { type: 'downloadFromUrl', data: { dataUrl: '/API/getStructure' } });

            if (portalList.length) {
              /** Socket Loja */
              this.io.to(portalList).emit('login', {
                text: 'UNIDADE CONECTADA',
                error: false,
                message: {
                  data: this.slavelist,
                },
                result,
              });
            }

            /**
             * Socket Loja
             * Monitoramento das vendas
             * Faz a solicitação para o Slaver
            */
            // Executar a primeira vez mais rápido quando abrir o Grid
            setTimeout(() => {
              console.log('Loja Vendas: ', new Date().getSeconds());
              socket.emit("saleSlave", data, socket.rooms);
            }, this.defaultInitTimeout);

            this.intervals.push({
              id: socket.id,
              interval: setInterval(() => {
                // console.log('Monitoramento', socket.id, socket.rooms, new Date().getSeconds());
                console.log('Loja Vendas: ', new Date().getSeconds());
                socket.emit("saleSlave", data, socket.rooms);
              }, intervalMonitoring(data))
            });
            socket.on('saleSlaveError', (vendasError) => {
              console.log('Loja resposta de Vendas: ', new Date().getSeconds());
              /** Atualiza o this.slavelist */
              this.slavelist.filter((filial) => {
                if(filial.id === vendasError[0].FILIAL){
                  if(vendasError.length === 1) {
                    Object.assign(filial, {vendas: vendasError[0].vendas});
                  }else {
                    Object.assign(filial, {vendasError});
                  }
                }
                return filial;
              });
              this.vendasError = vendasError;
            });


            /**
             * Socket Loja
             * Monitoramento dos Sats
             * Faz a solicitação para o Slaver
            */
            if (this.slavelist[subsidiaryIndex].sat !== 'NOT_SAT') {
              // Executar a primeira vez mais rápido quando abrir o Grid
              setTimeout(() => {
                console.log('Loja Sat: ', new Date().getSeconds());
                socket.emit("satSlave", data, socket.rooms);
              }, this.defaultInitTimeout);
              
              this.intervals.push({
                id: socket.id,
                interval: setInterval(() => {
                  // this.io.to(portalList).emit('satSlave', data);
                  console.log('Loja Sat: ', new Date().getSeconds());
                  socket.emit("satSlave", data, socket.rooms);
                }, intervalMonitoring(data))
              });
              socket.on('satSlaveError', (satError) => {
                console.log('Loja resposta do Sat: ', new Date().getSeconds());
                /** Atualiza o this.slavelist */
                this.slavelist.filter((filial) => {
                  if(filial.id === satError[0].id){
                    if(satError.length === 1) {
                      Object.assign(filial, {sat: satError[0].sat});
                    }else {
                      Object.assign(filial, {satError});
                    }
                  }
                  return filial;
                });
                this.satError = satError;
              });
            }


            /**
             * Socket Loja
             * Monitoramento dos parametros
             * parameters
             */
            // Executar a primeira vez mais rápido quando abrir o Grid
            setTimeout(() => {
              console.log('Loja Parametros: ', new Date().getSeconds());
              socket.emit("parametersSlave", data, socket.rooms);
            }, this.defaultInitTimeout);

            this.intervals.push({
              id: socket.id,
              interval: setInterval(() => {
                // this.io.to(portalList).emit('parametersSlave', data);
                console.log('Loja Parametros: ', new Date().getSeconds());
                socket.emit("parametersSlave", data, socket.rooms);
              }, intervalMonitoring(data))
            });
            socket.on('parametersSlaveError', (parametersError) => {
              console.log('Loja resposta de parametros: ', new Date().getSeconds());
              /** Atualiza o this.slavelist */
              this.slavelist.filter((filial) => {
                let id;
                if(parametersError.length && parametersError[0].length){
                  id = parametersError[0].id;
                }else if(parametersError.length && parametersError[1].length){
                  id = parametersError[1].id;
                }

                if(filial.id === id){
                  Object.assign(filial, {parametersError});
                }
                return filial;
              });
              this.parametersError = parametersError;
            });
          }
        })
        .catch((error) => {
          this.io.to(socket.id).emit('message', 'Você não é um usuário autorizado!');
          socket.disconnect();
        });
    }
  }

  async getDashboardFiliais(data = null) {
    const filiais = [];
    const queryResultAll = await adqGetDashboardFiliais.get(paginacao(data));
    const { rows } = queryResultAll;

    rows.forEach((element) => {
      filiais.push({
        id: element[0],
        codigo: element[0],
        name: element[1],
        situacao: 'Inativa',
        email: '',
        socketId: '',
        sat: element[3] == 0 ? 'NOT_SAT' : false,
        vendas: false,
        delivery: false,
        websocket: '',
        nrorg: element[2],
      });
    });

    if (this.slavelist !== undefined) { filiais.concat(this.slavelist); }

    return filiais;
  }
}

module.exports = new SocketService();
